<?php

namespace Drupal\ldap_auth;

class AuditAndLogs {

  private $username;
  private $time;
  private $error_occured;
  private $user_email = '';

  const EMAIL_NOT_RECEIVED = "EMAIL NOT RECEIVED";
  const USER_NOT_EXIST_IN_DRUPAL = "USER NOT EXIST IN DRUPAL";
  const USER_NOT_EXIST_IN_LDAP = "USER NOT EXIST IN LDAP";
  const WRONG_PASSWORD = "WRONG PASSWORD";
  const BLOCKED_USER = "BLOCKED USER";
  const LDAP_NOT_RESPONDING = "LDAP NOT RESPONDING";
  const USER_LOGGED_IN_USING_DRUPAL_CREDENTIALS = "USER LOGGED IN USING DRUPAL CREDENTIALS";

  public function __construct($username,$time,$error_occured,$user_email='') {
    $this->username = $username;
    $this->time = $time;
    $this->error_occured = $error_occured;
    $this->user_email = $user_email;
  }

  public function getPossibleSolution(){

    global $base_url;
    $email_attribute = \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_email_attribute');
    $ldap_connect = new LDAPFlow();
    $search_filter = $ldap_connect->getSearchFilter();
    $search_bases = $ldap_connect->getSearchBase();

    switch ($this->error_occured) {

      case "EMAIL NOT RECEIVED":
        return "Email address not received in the <b>" . $email_attribute . " </b> attribute of LDAP for the user <i>" . $this->username . "</i>. Please check your <a  href='" . $base_url . "/admin/config/people/ldap_auth/attribute_mapping' target='_blank'>email attribute mapping</a> .";
        break;
      case "USER NOT EXIST IN DRUPAL" :
        return "User found in LDAP server but no account with <u><i>username = ". $this->username ."</i></u> or <u><i>email = ".$this->user_email ." </i></u> exist in Drupal database. Please enable the <b>auto create feature</b> under the <i>LDAP Configuration</i> tab or <a href='".$base_url."/admin/people/create' target='_blank'>click here</a> to add the user in drupal manually.";
        break;
      case "USER NOT EXIST IN LDAP":
        return "User with <i>" . $search_filter . " = " . $this->username . "</i> not found under <b>" . $search_bases . "</b>. Please check if the user exist in your selected search base.";
        break;
      case "WRONG PASSWORD":
        return "User found in LDAP server but password does not match.";
        break;
      case "BLOCKED USER":
        return "Cannot login the user (" . $this->username . ") as it is blocked in Drupal.";
        break;
      case 'LDAP NOT RESPONDING':
        return "This error could come due to multiple reasons. Please configure your LDAP configurations again or contact your LDAP administrator.";
        break;
      case 'USER LOGGED IN USING DRUPAL CREDENTIALS':
        return "User logged in using Drupal credentials.";
        break;
      default:
        echo "Error occurred";

    }
  }

  public function addAudits(){

    $values = [
     $this->username,
     $this->time,
     $this->error_occured,
     $this->user_email,
    ];

    try {
      $connection = \Drupal::database();
      $connection->insert('mo_ldap_audits_and_logs')
        ->fields(['user','date','error','mail'], $values)
        ->execute();
    } catch (Exception $exception) {
      LDAPLOGGERS::addLogger($exception->getMessage());
      return null;
    }
  }

  public function getUserEmail(){
    return $this->user_email;
  }

}
